#!/usr/bin/env python

import os, sys
topdir = os.path.abspath(os.path.dirname(__file__))

# gettext installation
import gettext
gettext.install("dsgui", os.path.join(topdir, "locale"))

# import and check gtk
GTK_REQUIRE = "2.0"
try:
  import pygtk
  pygtk.require("2.0") 
except:
  print _("Required PyGTK 2.0 was not found - try installing python-gtk, pygtk or similar package.")
  sys.exit()
try:
  import gtk
except:
  print _("Python module 'gtk' was not found - try installing python-gtk, pygtk or similar package.")

GTK_REQUIRED_MINIMUM = (2,16,0)
check = gtk.check_version(*GTK_REQUIRED_MINIMUM)
if check: # check did not pass
  current_version = gtk.gtk_version
  message = _("""Your version of GTK is older that the required minimum.
The application will continue, but it will probably not work properly.

Information about versions:
  Your version: %d.%d.%d
  Required version: %d.%d.%d
  Result of check: %s""") % (current_version+GTK_REQUIRED_MINIMUM+(check,))
  d = gtk.MessageDialog(None, message_format=message, buttons=gtk.BUTTONS_OK, type=gtk.MESSAGE_WARNING)
  d.run()
  d.destroy()
  
try:
  import gnomevfs
  GNOMEVFS_AVAILABLE = True
except:
  GNOMEVFS_AVAILABLE = False
import pango

from dslib.client import Client
from ds_search import DsSearchDialog
from ds_send import DsSendDialog
from xml_translator import translate_xml


class GUI(object):

  topdir = topdir

  date_time_format = "%Y-%m-%d %H:%M"

  def __init__(self):
    self.username = None
    self.password = None
    self.test_account = None
    self.received_messages = {} # id2message
    self.sent_messages = {}
    self.message_data = {} # id to data
    # build the GUI from xml
    builder = gtk.Builder()
    #builder.set_translation_domain("dsgui")
    self.builder = builder
    gui_xml = GUI.translate_ui_xml("dsgui.ui")
    try:
      builder.add_from_string(gui_xml.decode('utf-8'))
    except:
      # this fixes problem with different API on windows and PyGTK 2.12
      builder.add_from_string(gui_xml.decode('utf-8'), -1)
    #builder.add_from_file("dsgui.xml")
    builder.connect_signals(self)
    self.window1 = builder.get_object("window1")
    self.window1.show()
    tag_table = builder.get_object("message_tagtable")
    self.received_message_store = builder.get_object("received_message_store")
    self.received_message_treeview = builder.get_object("received_message_treeview")
    self.received_message_textview = builder.get_object("received_message_textview")
    self.received_attachment_treeview = builder.get_object("received_attachment_treeview")
    self.received_attachment_store = builder.get_object("received_attachment_store")
    self.sent_message_store = builder.get_object("sent_message_store")
    self.sent_message_treeview = builder.get_object("sent_message_treeview")
    self.sent_message_textview = builder.get_object("sent_message_textview")
    self.statusbar = builder.get_object("statusbar")
    self._download_context = self.statusbar.get_context_id("download")
    # the following should not be necessary, but Glade somehow does not allow
    # to populate a tagtable with tags
    tag_table = builder.get_object("message_tagtable")
    tag_table.add(builder.get_object("bold_texttag"))
    tag_table.add(builder.get_object("larger_texttag"))
    # for some reason, "message_tagtable" is associated only with the "sent" buffer,
    # not with "received" - I fix it here by hand
    self.received_message_textview.get_buffer().create_tag("bold", weight=pango.WEIGHT_BOLD)
    self.received_message_textview.get_buffer().create_tag("larger", scale=1.5, pixels_above_lines=10)
    # some tweaks to permit nice display of read messages
    col = builder.get_object("received_message_read_column")
    cell = builder.get_object("received_message_read_cell_renderer")
    col.set_cell_data_func(cell, self._render_read_icon)
    # add sorting info by hand - it does not load properly from XML
    # in older version of gtk (2.16 and below)
    for colname, i in (("treeviewcolumn_id", 0),("treeviewcolumn_title",1),
                       ("treeviewcolumn_sender",2), ("treeviewcolumn7",3),
                       ("treeviewcolumn8",4), ("received_message_read_column",5),
                       ("treeviewcolumn3",0), ("treeviewcolumn4",1),
                       ("treeviewcolumn5",2), ("treeviewcolumn6",6),
                       ("treeviewcolumn9",4), ("treeviewcolumn10",5),
                       ):
      col = builder.get_object(colname)
      col.set_sort_column_id(i)
    # bind text boldness to read locally status for received_attachment_treeview
    for colname in ("treeviewcolumn_id","treeviewcolumn_title","treeviewcolumn_sender",
                    "treeviewcolumn7","treeviewcolumn8"):
      col = builder.get_object(colname)
      renderer = col.get_cell_renderers()[0]
      col.set_cell_data_func(renderer, self._render_read_text)
    # load a stored config file
    self._load_config_file()
    # open an account
    if not self.username:
      self.on_edit_credentials_activate(self)
    self._check_empty_message_stores()

    
  # Callbacks
  
  def on_connect_action_activate(self, widget):
    received_count, sent_count = 0, 0
    # get received messages
    reply, error = self.try_client_method(
      self.client.GetListOfReceivedMessages, (),
      error_message=_("It was not possible to download the list of received messages.")
      )
    if not error:
      if reply.status.dmStatusCode == "0000":
        for m in reply.data:
          self.add_received_message(m)
          received_count += 1
      # get sent messages
      reply, error = self.try_client_method(
        self.client.GetListOfSentMessages, (),
        error_message=_("It was not possible to download the list of sent messages.")
        )
      if not error:
        if reply.status.dmStatusCode == "0000":
          for m in reply.data:
            self.add_sent_message(m)
            sent_count += 1
    self.statusbar.push(self._download_context,
                        _("%d received and %d sent messages found on server")%(received_count,sent_count))
    self._check_empty_message_stores()


  def on_about_activate(self, widget):
    dial = self.builder.get_object("aboutdialog1")
    dial.run()
    dial.hide()

  def on_received_message_treeview_cursor_changed(self, treeview):
    message = self._get_selected_received_message()
    if message:
      self.show_received_message(message)
      # scroll to selection - it can disappear when sorting by read locally status
      # and reading something
      model, paths = self.received_message_treeview.get_selection().get_selected_rows()
      if len(paths) == 1:
        self.received_message_treeview.scroll_to_cell(paths[0])
    self.on_received_attachment_treeview_cursor_changed(treeview)


  def on_window1_destroy(self, w):
    self._save_state()
    gtk.main_quit()


  def on_download_complete_message_activate(self, w):
    message = self._get_selected_received_message()
    reply, error = self.try_client_method(
      self.client.MessageDownload,
      (message.dmID,),
      error_message=_("It was not possible to download the message."),
      )
    if not error:
      message = reply.data
      self.add_received_message(message)
      self.show_received_message(message)
      self.on_received_attachment_treeview_cursor_changed(None)

  def on_received_attachment_treeview_row_activated(self, tree, path, column):
    self.builder.get_object("open_attachment").activate()

  def on_save_attachment_activate(self, w):
    message, attachment = self._get_selected_attachment()
    if attachment:
      d = gtk.FileChooserDialog(title=_("Select where to save"),
                                parent=self.window1,
                                action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                buttons=(gtk.STOCK_OK,gtk.RESPONSE_ACCEPT,
                                         gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT)
                                )
      d.set_current_name(attachment._dmFileDescr)
      ok = d.run()
      if ok == gtk.RESPONSE_ACCEPT:
        import os
        fullname = d.get_filename()
        dir, name = os.path.split(fullname)
        attachment.save_file(dir, name)
      else:
        pass
      d.destroy()

  def on_open_attachment_activate(self, w):
    message, attachment = self._get_selected_attachment()
    done = False
    if GNOMEVFS_AVAILABLE:
      mime_type = gnomevfs.get_mime_type_for_data(attachment.get_decoded_content())
      mime_app = gnomevfs.mime_get_default_application(mime_type)
      if mime_app:
        temp_file = self._create_temp_file_for_dmfile(attachment)
        import subprocess
        application = subprocess.Popen([mime_app[2],temp_file])
        done = True
    if not done:
      d = gtk.MessageDialog(self.window1,
                            message_format="I could not find an application to open this attachment. You can save it and open it manually.",
                            buttons=gtk.BUTTONS_OK,
                            type=gtk.MESSAGE_ERROR)
      d.run()
      d.destroy()
      

  def on_sent_message_treeview_cursor_changed(self, treeview):
    message = self._get_selected_sent_message()
    self.show_sent_message(message)

    
  def on_edit_credentials_activate(self, w):
    d = self.builder.get_object("credentials_dialog")
    login_entry = self.builder.get_object("login_entry")
    login_entry.set_text(self.username or "")
    password_entry = self.builder.get_object("password_entry")
    password_entry.set_text(self.password or "")
    test_account_button = self.builder.get_object("test_environment_checkbutton")
    if self.test_account == None:
      # make test account the default 
      test_account_button.set_active(True)
    else:
      test_account_button.set_active(self.test_account)
    result = d.run()
    if result == 1:
      username = login_entry.get_text()
      password = password_entry.get_text()
      test_account = test_account_button.get_active()
      self._open_account(username, password, test_account)
    else:
      pass # cancel or something like this was pressed
    d.hide() # hide, don't destroy - the dialog is created by Builder


  def on_notebook1_switch_page( self, w, page, page_num):
    if page_num != 0:
      self.builder.get_object("download_complete_message").set_sensitive(False)
      self.builder.get_object("save_attachment").set_sensitive(False)
      self.builder.get_object("open_attachment").set_sensitive(False)
    else:
      self.on_received_message_treeview_cursor_changed(None)


  def on_find_data_box_activate(self, w):
    d = DsSearchDialog(self, self.client)
    result = d.run()
    d.hide()
    if result == 1:
      boxes = d.get_selected_box_names()
      m = "\n".join(["%s: %s"%(box[0],box[1]) for box in boxes])
      d = gtk.MessageDialog(self.window1,
                            message_format=_("You selected the following boxes:\n\n")+m,
                            buttons=gtk.BUTTONS_OK,
                            type=gtk.MESSAGE_INFO)
      d.run()
      d.destroy()
    
  def on_send_message_activate(self, w):
    dial = DsSendDialog(self, self.client)
    ok = False
    new = True
    while not ok:
      if not new:
        dial.show_all()
      new = False
      result = dial.run()
      dial.hide()
      if result == 1:
        # dummy operation at first to get a cookie
        reply, error = self.try_client_method(self.client.DummyOperation, (), handle_status=False)
        if not error:
          # now send the file
          error_message = _("It was not possible to send the message")
          envelope, dmfiles = dial.get_prepared_message()
          reply, error = self.try_client_method(self.client.CreateMessage,
                                                 (envelope, dmfiles),
                                                 error_message=error_message,
                                                 handle_status=False)
          message = reply.status.dmStatusMessage
          code = reply.status.dmStatusCode
          if code != "0000":
            mtype = gtk.MESSAGE_ERROR
            message_text = _("An error occured when sending the message:")
          else:
            mtype = gtk.MESSAGE_INFO
            message_text = _("Message was successfully sent:")
            ok = True
          message_text += "\n\n"
          message_text += _("Reply status: %(message)s\nReply code: %(code)s")%{"message":message,"code":code}
          d = gtk.MessageDialog(self.window1,
                                message_format=message_text,
                                buttons=gtk.BUTTONS_OK,
                                type=mtype)
          d.run()
          d.destroy()
      else:
        break

  def on_received_message_treeview_row_activated(self, tree, path, column):
    read_col = self.builder.get_object("received_message_read_column")
    if column == read_col:
      model = tree.get_model() 
      iter = model.get_iter(path) 
      msgid = model.get_value(iter, 0)
      read = self.message_data[msgid]['read_locally']
      read = not read
      self.message_data[msgid]['read_locally'] = read
      model.set_value(iter, 5, read)

  def _something_with_attachment_happened(self):
    message, attachment = self._get_selected_attachment()
    if attachment:
      self.builder.get_object("save_attachment").set_sensitive(True)
      self.builder.get_object("open_attachment").set_sensitive(True)
    else:
      self.builder.get_object("save_attachment").set_sensitive(False)
      self.builder.get_object("open_attachment").set_sensitive(False)
    
      
  def on_received_attachment_store_row_number_changed(self, model, path, iter=None):
    self._something_with_attachment_happened()

  def on_received_attachment_treeview_cursor_changed(self, treeview):
    self._something_with_attachment_happened()

#  def on_received_message_store_row_deleted(self, model, path):
#    self._check_empty_message_stores()
#    
#  def on_received_message_store_row_inserted(self, model, path, iter):
#    self._check_empty_message_stores() 
#  
#  def on_sent_message_store_row_deleted(self, model, path):
#    self._check_empty_message_stores()
#    
#  def on_sent_message_store_row_inserted(self, model, path, iter):
#    self._check_empty_message_stores()   

  # // end of Callbacks

  # helper methods for display

  def _render_read_icon(self, column, cell, model, iter):
    data = model.get_value(iter, 5)
    if data == True:
      stock = gtk.STOCK_YES
    else:
      stock = gtk.STOCK_NO
    pixbuf = self.window1.render_icon(stock_id=stock, size=gtk.ICON_SIZE_MENU)
    cell.set_property('pixbuf', pixbuf)

  def _render_read_text(self, column, cell, model, iter):
    data = model.get_value(iter, 5)
    if data == True:
      cell.set_property('weight', pango.WEIGHT_NORMAL)
    else: 
      cell.set_property('weight', pango.WEIGHT_BOLD)


  def _fill_message_data_into_textbuffer(self, message, textbuffer, additional_lines=None):
    def add_row(heading, text, add_tags=None):
      add = add_tags or ()
      position = textbuffer.get_end_iter()
      textbuffer.insert_with_tags_by_name(position, heading, "bold", *add)
      position = textbuffer.get_end_iter()
      textbuffer.insert_with_tags_by_name(position, " %s\n"%text, *add)
    def add_newline():
      position = textbuffer.get_end_iter()
      textbuffer.insert_with_tags_by_name(position, "\n")  

    # fill the text buffer
    textbuffer.set_text("")
    add_row(_("Subject:"), message.dmAnnotation, add_tags=("larger",))
    for name, value in (("dmLegalTitleLaw",_("Law")),
                        ("dmLegalTitleYear",_("Year")),
                        ("dmLegalTitleSect",_("Section")),
                        ("dmLegalTitlePar",_("Paragraph")),
                        ("dmLegalTitlePoint",_("Letter"))):
      if getattr(message, name):
        add_row(value+":", getattr(message, name))
    add_newline()
    add_row(_("From:"), message.dmSender)
    add_row(_("Sender Address:"), message.dmSenderAddress)
    add_newline()
    add_row(_("To:"), message.dmRecipient)
    add_row(_("Recipient Address:"), message.dmRecipientAddress)
    add_newline()
    add_row(_("Delivery time:"), self._format_datetime(message.dmDeliveryTime))
    add_row(_("Acceptance time:"), self._format_datetime(message.dmAcceptanceTime))
    add_newline()
    add_row(_("Status:"), "%d - %s" % (message.dmMessageStatus, message.get_status_description()))
    add_newline()
    if additional_lines:
      for head, text in additional_lines:
        add_row(head, text)
    

  # // end of helper methods for display


  def show_received_message(self, message):
    if message:
      textview = self.received_message_textview
      attachment_store = self.received_attachment_store
      textbuffer = textview.get_buffer()
      if hasattr(message, 'dmFiles') and message.dmFiles:
        attachments = (_("Attachments:"), _("%d (downloaded and ready)")%len(message.dmFiles))
        self.builder.get_object("download_complete_message").set_sensitive(False)
      else:
        attachments = (_("Attachments:"), _("not downloaded yet, ~%s KB; use 'Download' to get them.")%message.dmAttachmentSize)
        self.builder.get_object("download_complete_message").set_sensitive(True)

      self._fill_message_data_into_textbuffer(message, textbuffer, additional_lines=[attachments])
      # show the attachments
      attachment_store.clear()
      for i,f in enumerate(message.dmFiles):
        attachment_store.append([i,f._dmFileDescr,f.get_size(),f._dmMimeType,f._dmFormat])
      # mark message read
      mid = int(message.dmID)
      if mid in self.message_data:
        if not self.message_data[mid]['read_locally']:
          self.message_data[mid]['read_locally'] = True
          for row in self.received_message_store:
            if row[0] == mid:
              row[5] = True
              break


  def show_sent_message(self, message):
    if message:
      textview = self.sent_message_textview
      textbuffer = textview.get_buffer()
      textbuffer.set_text("")
      self._fill_message_data_into_textbuffer(message, textbuffer)
      

  def add_received_message(self, message):
    msgid = int(message.dmID)
    if msgid not in self.message_data:
      self.message_data[msgid] = {'read_locally': False}
    if msgid in self.received_messages:
      if self.received_messages[msgid].get_origin() == "tReturnedMessage":
        # this is a complete message - do not replace it
        # (it could be useful to create a list of increasing origin importance)
        pass
      else:
        self.received_messages[msgid] = message
    else:
      self.received_messages[msgid] = message
      delivery_time = self._format_datetime(message.dmDeliveryTime)
      accept_time = self._format_datetime(message.dmAcceptanceTime)
      read = self.message_data[msgid]['read_locally']
      data = [msgid,message.dmAnnotation,message.dmSender,delivery_time,accept_time,read]
      self.received_message_store.append(data)


  def add_sent_message(self, message):
    msgid = int(message.dmID)
    self.sent_messages[msgid] = message
    delivery_time = self._format_datetime(message.dmDeliveryTime)
    accept_time = self._format_datetime(message.dmAcceptanceTime)
    status = message.dmMessageStatus
    status_str = message.get_status_description()
    data = [msgid,message.dmAnnotation,message.dmRecipient,status_str,delivery_time,accept_time,status]
    for i,row in enumerate(self.sent_message_store):
      if row[0] == msgid:
        self.sent_message_store[i] = data
        break
    else:
      self.sent_message_store.append(data)

  def _get_selected_received_message(self):
    treeview = self.received_message_treeview
    model, paths = treeview.get_selection().get_selected_rows()
    if len(paths) == 1:
      message = self.received_messages[model[paths[0]][0]]
      return message
    return None
    
  def _get_selected_attachment(self):
    message = self._get_selected_received_message()
    if message:
      treeview = self.received_attachment_treeview
      model, paths = treeview.get_selection().get_selected_rows()
      if len(paths) == 1:
        attachment = message.dmFiles[model[paths[0]][0]]
        return message, attachment
      return message, None
    return None, None

  def _get_selected_sent_message(self):
    treeview = self.sent_message_treeview
    model, paths = treeview.get_selection().get_selected_rows()
    if len(paths) == 1:
      message = self.sent_messages[model[paths[0]][0]]
      return message
    return None
      

  def _save_state(self):
    # dump messages
    shelf = self._open_shelf()
    self._store_to_shelf(shelf)
    shelf.close()
    self._save_credentials()


  def _open_account(self, new_username, new_password, test_account):
    """called when the creadentials change - is responsible for cleaning up the UI,
    changing the ds_client and optionally loading stored messages for the new
    account"""
    if self.username == new_username and self.test_account == test_account:
      if self.password == new_password:
        return # we do not need to do anything, everything is the same
    else:
      # we need to replace the content of the UI
      shelf = self._open_shelf()
      if self.username:
        # we have a username, so we save the current state
        self._store_to_shelf(shelf)
      self.received_messages = {}
      self.sent_messages = {}
      self.message_data = {}
      self.username = new_username
      self.test_account = test_account
      self.sent_message_store.clear()
      self.received_message_store.clear()
      self.received_attachment_store.clear()
      self.received_message_textview.get_buffer().set_text("")
      self.sent_message_textview.get_buffer().set_text("")
      self.builder.get_object("download_complete_message").set_sensitive(False)
      self._load_from_shelf(shelf)
      shelf.close()
    self.password = new_password
    actions = ("connect_action", "find_data_box", "send_message")
    if self.username:
      self.client = self._create_ds_client(self.username, self.password, self.test_account)
      for action in actions:
        self.builder.get_object(action).set_sensitive(True)
    else:
      self.client = None
      for action in actions:
        self.builder.get_object(action).set_sensitive(False)
    self._check_empty_message_stores()


  def _open_shelf(self):
    import shelve
    shelf = shelve.open("messages.shelf", protocol=-1)
    return shelf


  def _store_to_shelf(self, shelf):
    key = self._get_shelf_key()
    shelf[key] = {"received_messages":self.received_messages,
                  "sent_messages":self.sent_messages,
                  "message_data":self.message_data,
                  }
    shelf.sync()


  def _load_from_shelf(self, shelf):
    data = shelf.get(self._get_shelf_key(), None)
    if data:
      message_data = data.get("message_data", None)
      if message_data:
        self.message_data = message_data
      for id,message in data['received_messages'].iteritems():
        self.add_received_message(message)
      for id,message in data['sent_messages'].iteritems():
        self.add_sent_message(message)

  def _get_shelf_key(self):
    """returns a key to be used for shelf based on current credentials"""
    return "%s___%s" % (self.username, self.test_account)


  def _show_error(self, message, exception=None):
    if exception:
      message += "\n\n"+_("Details:")+" "+str(exception)
    d = gtk.MessageDialog(self.window1,
                          message_format=message,
                          buttons=gtk.BUTTONS_OK,
                          type=gtk.MESSAGE_ERROR)
    d.run()
    d.destroy()

  def _handle_authorization_error(self, e, message_header=None):
    """convenience method to display information about failed authorization;
    returns True if the exception was handled, False otherwise"""
    if type(e.args) in (tuple,list) and \
    len(e.args) > 0 and type(e.args[0]) == tuple and len(e.args[0]) > 0:
      if e.args[0][0] == 401:
        # Unauthorized
        message = message_header and message_header+"\n\n" or ""
        message += _("Authorization failed!\n\nPlease check your credentials including the test-environment setting.")
        self._show_error(message)
        return True
    return False
  
  def _save_credentials(self):
    """save current creadentials to a file"""
    cr_filename = os.path.join(GUI.topdir, "dsgui.conf")
    cr_file = file(cr_filename, "wb")
    import ConfigParser
    config = ConfigParser.RawConfigParser()
    config.add_section('credentials')
    config.set('credentials', 'username', self.username)
    config.set('credentials', 'password', self.password)
    config.set('credentials', 'test_account', self.test_account)
    config.write(cr_file)
    cr_file.close()
    import stat
    os.chmod(cr_filename, stat.S_IREAD|stat.S_IWRITE)
                  
  def _load_config_file(self):
    cr_filename = self._get_config_filename()
    import ConfigParser
    config = ConfigParser.RawConfigParser()
    config.read(cr_filename)
    if config.has_option("credentials", "username"):
      username = config.get("credentials", "username")
    else:
      username = ""
    if config.has_option("credentials", "password"):
      password = config.get("credentials", "password")
    else:
      password = ""
    if config.has_option("password", "test_account"):
      test_account = config.getboolean("credentials", "test_account")
    else:
      test_account = True
    self._open_account(username, password, test_account)


  def _get_config_filename(self):
    return os.path.join(GUI.topdir, "dsgui.conf")
  
  def _check_empty_message_stores(self):
    """check if the store of received or sent messages is empty
    and warn if need be.
    """
    for treeview,message in ((self.received_message_treeview,[0,_("No received messages"),"","","",True]),
                             (self.sent_message_treeview,[0,_("No sent messages"),"","","","",""])):
      model = treeview.get_model()
      for i,row in enumerate(model):
        if row[0] == 0:
          model.remove(model.get_iter(i))
          break
      if len(model) == 0:
        model.append(message)
        treeview.set_sensitive(False)
      else:
        treeview.set_sensitive(True)


  def try_client_method(self, method, args, error_message=None, handle_status=True):
    """convenience function to run a method of self.client and display
    error messages, etc. as appropriate.
    Returns a tuple (reply, error_shown), where reply is the reply of the method and
    error_shown is a boolean telling if an error message was displayed.
    """
    if not error_message:
      error_message = _("An error occured during the following operation: %s") % method.__name__
    try:
      reply = method(*args)
    except Exception, e:
      if not self._handle_authorization_error(e, message_header=error_message):
        #print e
        self._show_error(error_message, exception=e)
      return None, True
    else:
      error_shown = False
      if handle_status:
        code, message = GUI._extract_code_and_message_from_reply(reply)
        if code != "0000":
          error_message += "\n\n"
          error_message += _("Reply status: %(message)s\nReply code: %(code)s")%{"message":message,"code":code}
          self._show_error(error_message)
          error_shown = True
      return reply, error_shown
      
  @classmethod
  def _extract_code_and_message_from_reply(cls, reply):
    if not reply:
      return None, None
    if not hasattr(reply, 'status'):
      return None, None
    elif hasattr(reply.status, 'dmStatusCode'):
      return reply.status.dmStatusCode, reply.status.dmStatusMessage
    elif hasattr(reply.status, 'dbStatusCode'):
      return reply.status.dbStatusCode, reply.status.dbStatusMessage
    return None, None

  @classmethod
  def _create_ds_client(self, username, password, test_account):
    return Client(username, password, test_environment=test_account)


  @classmethod
  def _create_temp_file_for_dmfile(self, dmfile):
    import tempfile
    ext = os.path.splitext(dmfile._dmFileDescr)[1]
    temp = tempfile.mkstemp(suffix=ext)
    os.write(temp[0], dmfile.get_decoded_content())
    os.close(temp[0])
    return temp[1]

  @classmethod
  def _format_datetime(cls, d):
    if d:
      return d.strftime(cls.date_time_format)
    return ""
  
  @classmethod
  def translate_ui_xml(cls, filename):
    gui_xml = translate_xml(os.path.join(cls.topdir,filename), _)
    return gui_xml


if __name__ == "__main__":
##   import logging
##   logging.basicConfig(level=logging.INFO)
##   logging.getLogger('suds.client').setLevel(logging.DEBUG)
  app = GUI()
  gtk.main()


# TODO
# - no credentials on start
#   - do not use default credentials, but rather store them somewhere in a config file
# - support multiple accounts
